const octRenderer = require('oct-renderer');
const templates = require('../templates');
const ipcRenderer = require('electron').ipcRenderer;
const oco = require('opencolor');

var showErrorTimer;

function updateErrorUI(err, title) {
  octRenderer.destroy($('[data-preview]')[0]);

  var msg = err;
  if (err.name === "ParserError") {
    msg = err.message.replace('Parse error on l', 'L');
  }

  title = title || 'Parse Error';

  const $previewError = templates.parserError({error: msg, title: title});
  $('[data-preview]').append($previewError);

  /*
  ipcRenderer.send('disableSaveMenuItem', true);
  $('button[data-save]').addClass('Button--disabled').prop('disabled', true);
  */
}

function displayRenderError(err, title) {
  updateErrorUI(err, title);
}

function displayError(err, delay) {
  if(!delay) {
    updateErrorUI(err);
  } else {
    delayError(err);
  }
}

function delayError(err, title) {
  clearTimeout(showErrorTimer);
  showErrorTimer = setTimeout(function() {
    updateErrorUI(err, title)
  }, 600);
}

function clearError() {
  clearTimeout(showErrorTimer);
  $('[data-error]').remove();
}

function update(currentDoc, delayError) {
  if (currentDoc.isValid) {
    updateValidDoc(currentDoc.tree)
  } else {
    displayError(currentDoc.parseError, delayError);
  }
}

function updateValidDoc(tree) {
  clearError();
  const $previewElm = $('[data-preview]');
  octRenderer.destroy($previewElm[0]);

  try {
    octRenderer.render(tree, $previewElm[0]);
  } catch(err) {
    console.error(err.stack)
    displayRenderError(err, 'Render Error')
  }
}

module.exports = {
  update: update,
  clearError: clearError,
  displayError: displayError
}
