const CodeMirror = require('../../vendor/codemirror/codemirror');
require('../../vendor/codemirror/dirty-state');
require('../../vendor/codemirror/active-line');
require('../../vendor/codemirror/oco-mode');

var onEditorChangedTimer;
var myCodeMirror;
var changeListener = function() {};

function onEditorChanged(codeMirror, changeObj) {
  clearTimeout(onEditorChangedTimer);
  onEditorChangedTimer = setTimeout(function() {
    const currentCodemirrorDoc = myCodeMirror.getDoc();

    currentCodemirrorDoc.markAsDirty();
    changeListener(currentCodemirrorDoc);
  }, 10);
}
function startPreviewMode() {
  myCodeMirror.setOption('readOnly', true);
  $('[data-editor]').addClass('is-readonly');
  return myCodeMirror.getDoc().getValue();
}

function endPreviewMode(restore) {
  myCodeMirror.setOption('readOnly', false);
  $('[data-editor]').removeClass('is-readonly');
  if(restore) {
    updateSilent(restore);
  }
}

function updateSilent(source) {
  myCodeMirror.off('change', onEditorChanged);
  myCodeMirror.getDoc().setValue(source);
  myCodeMirror.on('change', onEditorChanged);
}

function update(source, silent) {
  if(silent) {
    return updateSilent(source)
  }
  myCodeMirror.getDoc().setValue(source);
}

function setChangeListener(listener) {
  changeListener = listener;
}

function createDocument(data) {
  return CodeMirror.Doc(data, 'oco')
}

function changeFontSize(change) {
  const editorElm = $('.CodeEditor .CodeMirror');
  const fontSize = parseInt(editorElm.css('font-size')) + change;
  editorElm.css('font-size', fontSize + "px");
}

function getCodeMirror() {
  if (!myCodeMirror) {
    var config = {
      theme: "eclipse",
      dragDrop: false,
      styleActiveLine: true,
      lineNumbers: true,
    };

    myCodeMirror = CodeMirror($('[data-editor]')[0], config);
    myCodeMirror.setOption('extraKeys', {
      "Shift-Tab": "indentLess",
      "Tab": function(cm) {
        if (cm.somethingSelected()) {
          cm.indentSelection("add");
        } else {
          cm.replaceSelection(Array(cm.getOption("indentUnit") + 1).join(" "), "end", "+input");
        }
      }
    });
    myCodeMirror.on('change', onEditorChanged);
  }

  return myCodeMirror;
}

function init() {
  $(document).keydown(function(e) {
    if (e.metaKey && e.keyCode === 187) {
      changeFontSize(+2);
    }

    if (e.metaKey && e.keyCode === 189) {
      changeFontSize(-2);
    }
  });
}

function lock() {
  $('[data-editor-blur]').focus();
  myCodeMirror.setOption('readOnly', true);
}

function unlock() {
  myCodeMirror.setOption('readOnly', false);
  getCodeMirror().focus();
}

function focus() {
  getCodeMirror().focus();
}

module.exports = {
  getCodeMirror: getCodeMirror,
  setChangeListener: setChangeListener,
  createDocument: createDocument,
  startPreviewMode: startPreviewMode,
  endPreviewMode: endPreviewMode,
  update: update,
  lock: lock,
  unlock: unlock,
  focus: focus,
  init: init
}
