const templates = require('../templates');
const ocoTransformer = require('opencolor-transformer');


const validTransformFunctions = {
  'searchAndReplace': ' Search and Replace Names',
  'compoundWords': 'Transform Names',
  'abstractRepeating': 'Resolve Duplicates',
  'autoname': 'Name Colors',
  'flatten': 'Flatten Palettes',
  'group': 'Nest Palettes',
  'sort': 'Sort Colors'
}

var $panel = null;
var $functionOptions;
var currentTree = null;
var transformFunction = Object.keys(validTransformFunctions)[0];
var transformOptions = {};

var eventBus

function update(config) {
  var $transformPanelContent = createPanel(config);
  $panel.find('[data-panel-content]').html('').append($transformPanelContent);
}

function updateTree(tree, initialize) {
  currentTree = tree;

  performTransform();
}

function get(onClose, _eventBus) {
  eventBus = _eventBus;
  eventBus.sourcePreviewStateChanged(true);

  $panel = $(templates.transformTool({}));

  $panel.find('[data-transform-cancel]').on('click', function(e) {
    e.preventDefault();
    eventBus.sourcePreviewStateChanged(false);
    onClose();
  });

  $panel.find('[data-transform-apply]').on('click', function(e) {
    e.preventDefault();

    performTransform(function(transformedTree) {
      eventBus.sourcePreviewStateChanged(true);
      onClose();
    }, true);
  });

  return $panel;
}

function close() {
  eventBus.sourcePreviewStateChanged(false);
}

function updateFunctionOptionsUI(transformFunction, $optionsUI) {
  if ($optionsUI) {
    $functionOptions.html('').append($optionsUI);
  } else {
    const template = templates.loadTemplate('tools/Transform-' + transformFunction + '.hbs');
    const functionOptionsFormContent = '<div>' + template({}) + '</div>';
    $functionOptions.html(functionOptionsFormContent);
  }

  function readOptions() {
    var optionsArray = $functionOptions.serializeArray();
    var options = {};

    optionsArray.forEach(function(pair) {
      if (pair.value === "false") {
        options[pair.name] = false;
        return;
      }
      if (pair.value === "true") {
        options[pair.name] = true;
        return;
      }
      options[pair.name] = pair.value;
    })

    return options;
  }

  $functionOptions.find('input, select').unbind('keyup.transformFunction change.transformFunction');
  $functionOptions.find('input, select').bind('keyup.transformFunction change.transformFunction', function(e) {
    e.preventDefault();
    transformOptions = readOptions();
    performTransform();
  });
  transformOptions = readOptions();
  performTransform();
}

function createPanel(config) {

  // reset
  currentTree = null;
  transformFunction = Object.keys(validTransformFunctions)[0];
  transformOptions = {};

  if (config.transformFunction) {
    transformFunction = config.transformFunction;
  }

  var $optionsUI = config.$ui || false;

  var $panel = $(templates.transform({
    validTransformFunctions: validTransformFunctions
  }));

  $functionOptions = $panel.find('[data-transform-function-options]');
  $panel.find('[data-transform-select-function]').on('change', function(e) {
    e.preventDefault();
    transformFunction = $(this).val();
    updateFunctionOptionsUI(transformFunction);
  });

  updateFunctionOptionsUI(transformFunction, $optionsUI);
  $panel.find('[data-transform-select-function]').val(transformFunction);

  return $panel;
}

function getConfig() {
  return {
    transformFunction: transformFunction,
    transformOptions: transformOptions,
    $ui: ($functionOptions) ? $functionOptions.children().first() : null
  }
}

function showError(message) {
  $panel.find('[data-transform-apply]').prop('disabled', true);
  $panel.find('[data-transform-error]').text(message).show();
}

function hideError() {
  $panel.find('[data-transform-apply]').prop('disabled', false);
  $panel.find('[data-transform-error]').text('').hide();
}

function performTransform(callback, apply) {
  if (!currentTree) {
    showError('Not able to transform invalid palettes');
    return
  }

  ocoTransformer[transformFunction](currentTree, transformOptions).then(function(transformedTree) {
    hideError();
    eventBus.treeChanged(transformedTree, apply);
    if (callback) {
      callback(transformedTree);
    }
  }).catch((err) => {
    console.error(err.stack)
    showError(err);
  });
}


module.exports = {
  get: get,
  update: update,
  close: close,
  getConfig: getConfig,
  updateTree: updateTree
}
