const fs = require('fs-extra');
const path = require('path');
const fsevents = require('fsevents');
const config = require('./config');
const docsManager = require('./docs-manager');
const leftPane = require('./leftpane');


function copyItem(item, targetFolder) {
  const data = fs.readFileSync(item.path, 'utf8');
  const fileName = path.basename(item.path);
  const targetPath = path.join(targetFolder, fileName)

  fs.writeFileSync(targetPath, data, 'utf-8');
  fs.utimesSync(targetPath, item.stats.atime, item.stats.birthtime) // set mtime to birthtime so we can sort in the picker the same way as in the app
}

function isValidLibraryFile(filePath) {
  // we don't support nested folders yet, only look at files in the top level
  const dirName = path.dirname(filePath) + '/';
  const isRootFile = dirName === config.settings.get('libfolder')

  return isRootFile && config.isValidFile(filePath)
}

function syncToLib(path) {
  const sourceFolder = config.settings.get('libfolder');
  const targetFolder = config.paletteFolderPath;

  fs.removeSync(targetFolder);
  fs.mkdirpSync(targetFolder);

  const items = []
  fs.walk(sourceFolder)
    .on('data', function(item) {
      if (isValidLibraryFile(item.path)) {
        items.push(item)
      }
    })
    .on('end', function() {
      items.forEach(function(item) {
        copyItem(item, targetFolder)
      })
    })

  fs.writeFileSync(targetFolder + 'libfolder.setting', sourceFolder, 'utf-8');
}

function onLibFolderChange(path) {
  docsManager.syncDocs(path)
  leftPane.render()
  syncToLib(path)
}

function init() {
  const watcher = fsevents(config.settings.get('libfolder'));

  watcher.on('change', function(path, info) {
    if ((info.type === 'file' || info.type === 'symlink') && isValidLibraryFile(info.path)) {
      onLibFolderChange(info.path)
    }
  });

  watcher.start()
  syncToLib(path) // first time sync
}


exports.init = init;
