const fs = require('fs-extra')
const plist = require('plist')
const shell = require('electron').shell
const config = require('./config')
const utils = require('./utils')

//
// Picker

function isPickerInstalled () {
  return utils.pathIsReadable(config.colorPickerPath)
}

function getPickerVersion (path) {
  const infoPlistData = fs.readFileSync(path + '/Contents/Info.plist', 'utf8')
  const infoPlist = plist.parse(infoPlistData)

  return infoPlist.CFBundleShortVersionString
}

function isPickerUpToDate () {
  installedVersion = getPickerVersion(config.colorPickerPath)
  bundledVersion = getPickerVersion(__dirname + '/../resources/OpenColorPicker.colorPicker')

  return (installedVersion === bundledVersion)
}

function checkForLatestPicker () {
  // if (isPickerInstalled() && isPickerUpToDate()) {
  //   console.log('Most recent version of colorPicker already installed')
  // } else {
  installPicker()
  // }
}

function installPicker () {
  fs.accessSync(config.colorPickersFolderPath, fs.R_OK | fs.W_OK)

  const localColorPicker = fs.realpathSync(__dirname + '/../resources/OpenColorPicker.colorPicker')

  fs.copy(localColorPicker, config.colorPickerPath, (err) => {
    if (err) throw err
    console.log('Installed new colorPicker')
  })
}

//
// Example Palettes

function installSamplePalettes (libFolderPath) {
  const examples = ['Apple UI Colors.oco', 'Material Design.oco', 'Open Color Views.oco', 'Open Color View Options.oco', 'Reader.oco']

  examples.forEach(function (name) {
    var examplePalette = fs.realpathSync(__dirname + '/../resources/' + name)
    fs.copySync(examplePalette, libFolderPath + name)
  })
}

//
// Sketch Plugin

function isSketchPluginInstalled () {
  return utils.pathIsReadable(config.sketchPluginPath)
}

function getSketchPluginVersion (path) {
  pkgJson = utils.readJsonFile(path + '/Contents/Sketch/manifest.json')
  return pkgJson.version
}

function isSketchPluginUpToDate () {
  installedVersion = getSketchPluginVersion(__dirname + '/../resources/opencolortools.sketchplugin')
  bundledVersion = getSketchPluginVersion(config.sketchPluginPath)

  return (installedVersion === bundledVersion)
}

function installSketchPlugin () {
  const localSketchPlugin = fs.realpathSync(__dirname + '/../resources/opencolortools.sketchplugin')

  if (!utils.pathIsWritable(config.sketchPluginsFolderPath)) {
    console.log("Can't write to sketch plugin folder")
    shell.openItem(localSketchPlugin)
  } else {
    fs.copy(localSketchPlugin, config.sketchPluginPath, (err) => {
      if (err) throw err
      console.log('Installed new Sketch plugin')
    })
  }
}

function installPickerHelperPlugin () {
  const localPickerHelperPlugin = fs.realpathSync(__dirname + '/../resources/PickerHelper.sketchplugin')

  if (!utils.pathIsWritable(config.sketchPluginsFolderPath)) {
    console.log("Can't write to sketch plugin folder")
    shell.openItem(localPickerHelperPlugin)
  } else {
    fs.copy(localPickerHelperPlugin, config.sketchPickerHelperPluginPath, (err) => {
      if (err) throw err
      console.log('Installed new Sketch plugin')
    })
  }
}

function checkForLatestSketchPlugin () {
  // if (isSketchPluginInstalled()) {
  //   if (isSketchPluginUpToDate()) {
  //     console.log('Most recent version of Sketch plugin already installed')
  //   } else {
  installSketchPlugin()
  //   }
  // }
}

function isLibFolderSet () {
  const libfolder = config.settings.get('libfolder')
  return libfolder && utils.pathIsWritable(libfolder)
}

function installCompleted () {
  return isLibFolderSet()
}

function ensureEverythingUpToDate () {
  checkForLatestPicker()
  checkForLatestSketchPlugin()
  installPickerHelperPlugin()
}

exports.installCompleted = installCompleted
exports.installSketchPlugin = installSketchPlugin
exports.installSamplePalettes = installSamplePalettes
exports.ensureEverythingUpToDate = ensureEverythingUpToDate
