const fs = require('fs-extra');
const handlebars = require('../vendor/handlebars');
const utils = require('./utils')

function loadTemplate(filename) {
  const templateData = fs.readFileSync(__dirname + '/../templates/' + filename, 'utf8');
  return handlebars.compile(templateData);
}

function loadOcoTemplate(name) {
  const path = __dirname + '/../templates/oco/' + name + '.oco';
  if(!utils.pathIsReadable(path)) {
    return false;
  }
  return fs.readFileSync(path, 'utf8');
}

handlebars.registerHelper('conditionalClass', function(bool, className) {
  if (bool) {
    return className
  } else {
    return ''
  }
})

module.exports = {
  documentListItem: loadTemplate('DocumentListItem.hbs'),
  parserError: loadTemplate('ParserError.hbs'),
  onboarding: loadTemplate('Onboarding.hbs'),

  renamePaletteModal: loadTemplate('modals/RenamePalette.hbs'),
  dropzoneModal: loadTemplate('modals/dropzone.hbs'),
  selectFileModal: loadTemplate('modals/selectFile.hbs'),
  importModal: loadTemplate('modals/import.hbs'),

  importTool: loadTemplate('tools/ImportTool.hbs'),
  transformTool: loadTemplate('tools/TransformTool.hbs'),
  exportTool: loadTemplate('tools/ExportTool.hbs'),
  shareTool: loadTemplate('tools/ShareTool.hbs'),
  transform: loadTemplate('Transform.hbs'),
  assistant: loadTemplate('DocumentAssistant.hbs'),

  loadTemplate: loadTemplate,
  loadOcoTemplate: loadOcoTemplate
};
