const fs = require('fs-extra');
const path = require('path');
const child_process = require('child_process');
const config = require('./config');
const leftPane = require('./leftpane');


function checkPath(path, access) {
  try {
    fs.accessSync(path, access);
    return true;
  } catch (e) {
    return false;
  }
}

function pathIsReadable(path) {
  return checkPath(path, fs.R_OK);
}

function pathIsWritable(path) {
  return checkPath(path, fs.R_OK | fs.W_OK);
}

function ensurePath(path) {
  fs.mkdirpSync(path);
}

function readJsonFile(filePath) {
  if (!pathIsReadable(filePath)) {
    return null
  }

  var jsonData = null
  const jsonString = fs.readFileSync(filePath, 'utf-8')

  try {
    jsonData = JSON.parse(jsonString);
  } catch (e) {
  }

  return jsonData
}

function deleteFile(filePath) {
  const fileInfo = fs.statSync(filePath);

  if (fileInfo.isFile() && config.isValidFile(filePath)) {
    child_process.execSync(`osascript -e 'tell app "Finder" to move the POSIX file "${filePath}" to trash'`)
  }
}

function getNameFromPath(filePath) {
  const fileName = path.basename(filePath)
  const name = path.basename(filePath, ".oco")

  return name
}

function getAvailableTargetPath(name) {
  var targetPath = path.join(config.settings.get('libfolder'), `${name}.oco`);
  if (!pathIsReadable(targetPath)) {
    return targetPath
  }

  for (var i = 2; i < 500; i++) {
    targetPath = path.join(config.settings.get('libfolder'), `${name}-${i}.oco`);
    if (!pathIsReadable(targetPath)) {
      return targetPath
    }
  }

  throw "can't generate valid targetPath"
}

function symlinkFile(filePath) {
  const targetPath = getAvailableTargetPath(getNameFromPath(filePath))

  leftPane.setCurrentFile(targetPath);
  fs.symlinkSync(filePath, targetPath);
}

function copyFile(filePath) {
  const targetPath = getAvailableTargetPath(getNameFromPath(filePath))

  leftPane.setCurrentFile(targetPath);
  fs.copySync(filePath, targetPath);
}

function writeFile(name, data) {
  const targetPath = getAvailableTargetPath(name)

  leftPane.setCurrentFile(targetPath);
  fs.writeFileSync(targetPath, data, 'utf-8');
}

function readFile(filePath, encoding) {
  return fs.readFileSync(filePath, encoding);
}


exports.ensurePath = ensurePath;
exports.pathIsReadable = pathIsReadable;
exports.pathIsWritable = pathIsWritable;
exports.getNameFromPath = getNameFromPath;

exports.copyFile = copyFile;
exports.readFile = readFile;
exports.writeFile = writeFile;
exports.deleteFile = deleteFile;
exports.symlinkFile = symlinkFile;

exports.readJsonFile = readJsonFile;
