import { Component } from 'react'

import { colorIsDark, colorIsVeryLight } from '../../lib/luminance'
import { RenderError } from '../../lib/oco-utils'

export default class BaseComponent extends Component {
  constructor (props) {
    super(props)

    this.renderEntry = this.renderEntry.bind(this)
    this.renderReference = this.renderReference.bind(this)
  }

  getClassesForColor (color, styles, index) {
    let classes = [styles.Color]

    if (colorIsDark(color.hexValue)) {
      classes.push('OctColor__dark')
      classes.push(styles.Color__dark)
    } else {
      classes.push('OctColor__light')
      classes.push(styles.Color__light)

      let isVeryLight = colorIsVeryLight(color.hexValue)
      if (isVeryLight) {
        classes.push(styles.Color__veryLight)
      }
    }

    return classes.join(' ')
  }

  renderPalette (entry, index) {
    throw new RenderError('Sorry, nested palettes are not yet supported')
  }

  renderReference (item, index) {
    return this.renderColor(item, index)
  }

  renderEntry (item, index) {
    return {
      'Palette': this.renderPalette,
      'Color': this.renderColor,
      'Reference': this.renderReference
    }[item.type](item, index)
  }
}
