import React, { Component } from 'react'
import _ from 'lodash'

import styles from './ColorPicker.scss'

import MinimalView from '../views/MinimalView'
import CardsView from '../views/CardsView'
import SamplesView from '../views/SamplesView'
import BarsView from '../views/BarsView'
import ListView from '../views/ListView'
import GridView from '../views/GridView'
import SquaresView from '../views/SquaresView'
import BlockView from '../views/BlockView'
import StackView from '../views/StackView'
import FanView from '../views/FanView'

import { parseColor } from '../../lib/oco-utils'

const viewClassMapping = {
  'minimal': MinimalView,
  'cards': CardsView,
  'samples': SamplesView,
  'bars': BarsView,
  'list': ListView,
  'grid': GridView,
  'squares': SquaresView,
  'block': BlockView,
  'stack': StackView,
  'fan': FanView
}

export default class ColorPicker extends Component {
  constructor (props) {
    super(props)

    this.defaultView = 'list'

    this.renderEntry = this.renderEntry.bind(this)
    this.renderRoot = this.renderRoot.bind(this)
    this.renderColor = this.renderColor.bind(this)
    this.renderPalette = this.renderPalette.bind(this)
    this.renderReference = this.renderReference.bind(this)
  }

  parseViewOptions (block) {
    let options = {
      view: this.defaultView
    }
    block.metadata.keys().forEach(function (key) {
      if (/^oct\//i.test(key)) {
        let newKey = key.replace(/^oct\//i, '')
        let newValue = block.metadata.get(key)

        if (newValue.type === 'ColorValue') {
          options[newKey] = newValue.value
        } else if (newValue.type === 'Reference') {
          let targetBlock = newValue.resolved()
          if (targetBlock && targetBlock.type === 'Color') {
            let color = parseColor(targetBlock)
            options[newKey] = color.hexValue
          }
        } else {
          options[newKey] = newValue
        }
      }
    })
    return options
  }

  getViewClass (view) {
    if (viewClassMapping.hasOwnProperty(view)) {
      return viewClassMapping[view]
    } else {
      return ListView
    }
  }

  renderPalette (block, index) {
    if (!block.parent) {
      return this.renderRoot(block, index)
    }
    let options = {
      entries: block.children,
      name: block.name,
      options: this.parseViewOptions(block),
      key: index
    }

    return React.createElement(this.getViewClass(options.options.view.toLowerCase()), options)
  }

  renderRoot (block, index) {
    if (block.metadata.get('oct/defaultView')) {
      this.defaultView = block.metadata.get('oct/defaultView')
    } else {
      this.defaultView = 'list'
    }

    if (block.children.every(function (block) { return block.type === 'Color' || block.type === 'Reference' })) {
      block.name = ''
      block.parent = true
      return this.renderPalette(block)
    } else {
      return block.children.map(this.renderEntry)
    }
  }

  renderReference (block, index) {
    let targetBlock = block.resolved()
    if (targetBlock) {
      let target = _.cloneDeep(targetBlock)
      target.name = block.name
      target.metadata = block.metadata

      return this.renderEntry(target, index)
    } else {
      // TODO: how to render a invalid reference?
    }
  }

  renderColor (block, index) {
  }

  renderEntry (block, index) {
    return {
      'Palette': this.renderPalette,
      'Color': this.renderColor,
      'Reference': this.renderReference
    }[block.type](block, index)
  }

  render () {
    return (
      <div className={styles.ColorPicker} style={this.props.style}>
        {this.renderEntry(this.props.parsedOco)}
      </div>
    )
  }
}

ColorPicker.propTypes = {
  parsedOco: React.PropTypes.object
}

ColorPicker.defaultProps = {
  parsedOco: {}
}
