import React from 'react'
import BaseComponent from '../BaseComponent/BaseComponent'
import styles from './Flex.scss'
import { colorIsDark } from '../../lib/luminance'
import { parseColor } from '../../lib/oco-utils'

export default class Flex extends BaseComponent {
  constructor (props) {
    super(props)

    this.options = props.options

    this.renderColor = this.renderColor.bind(this)
    this.renderPalette = this.renderPalette.bind(this)
  }

  renderColorInfo (color) {
    let showName = this.options.showName
    let showValue = this.options.showValue

    let colorInfo = ''
    let colorValue = ''

    if (showName) {
      colorInfo = <div className={styles.Color_info_name}>{color.name}</div>
    }
    if (showValue) {
      colorValue = <div className={styles.Color_info_value}>{color.value}</div>
    }
    if (colorInfo || colorValue) {
      colorInfo =
        <div className={styles.Color_info}>
          {colorInfo}
          {colorValue}
        </div>
    }
    return colorInfo
  }

  renderColor (entry, index) {
    let color = parseColor(entry)

    var weight = color.options.weight || 1
    weight = Math.min(3, weight)
    weight = Math.max(1, weight)

    return (
      <div className={this.getClassesForColor(color, styles, index)} key={index} style={{flexGrow: weight}}>
        <div className={styles.Color_swatch + ' OctColor'}
          style={{backgroundColor: color.hexValue}}
          data-color-value={color.value}
          data-color-hexvalue={color.hexValue}
          data-color-name={color.name}></div>
        {this.renderColorInfo(color)}
      </div>
    )
  }

  getClassesForComponent () {
    let classes = [styles.Flex]

    if (this.options.swatchShape === 'SQUARE') {
      classes.push(styles.Flex__swatchShapeSquare)
    } else {
      classes.push(styles.Flex__swatchShapeRect)
    }

    let backgroundIsDark = colorIsDark(this.options.backgroundColor)
    if (backgroundIsDark) {
      classes.push(styles.Flex__dark)
    }

    return classes.join(' ')
  }

  render () {
    return (
      <div className={this.getClassesForComponent() + ' OctPalette'} data-palette-title={this.props.name} style={{backgroundColor: this.options.backgroundColor}}>
        <div className={styles.Flex_header}>{this.props.name}</div>

        <div className={styles.Flex_items}>
          {this.props.entries.map(this.renderEntry)}
        </div>
      </div>
    )
  }
};

Flex.propTypes = {
  entries: React.PropTypes.array.isRequired,
  name: React.PropTypes.string,
  options: React.PropTypes.shape({
    swatchShape: React.PropTypes.string,
    showName: React.PropTypes.bool,
    showValue: React.PropTypes.bool,
    backgroundColor: React.PropTypes.string
  })
}
