import React from 'react'
import BaseComponent from '../BaseComponent/BaseComponent'
import styles from './Float.scss'
import { colorIsDark } from '../../lib/luminance'
import { parseColor } from '../../lib/oco-utils'

export default class Float extends BaseComponent {
  constructor (props) {
    super(props)

    this.options = props.options

    this.renderColor = this.renderColor.bind(this)
    this.renderPalette = this.renderPalette.bind(this)
  }

  renderColorInfo (color) {
    let showName = this.options.showName
    let showValue = this.options.showValue

    let colorInfo = ''
    let colorValue = ''

    if (showName) {
      colorInfo = <div className={styles.Color_info_name}>{color.name}</div>
    }
    if (showValue) {
      colorValue = <div className={styles.Color_info_value}>{color.value}</div>
    }
    if (colorInfo || colorValue) {
      colorInfo =
        <div className={styles.Color_info}>
          {colorInfo}
          {colorValue}
        </div>
    }
    return colorInfo
  }

  renderColor (entry, index) {
    let color = parseColor(entry)

    let colorWidth

    if (!this.options.fixedSize) {
      colorWidth = {width: (100 / this.options.colorsPerRow) + '%'}
    }

    return (
      <div className={this.getClassesForColor(color, styles, index)} key={index} style={colorWidth}>
        <div className={styles.Color_swatch + ' OctColor'}
          style={{backgroundColor: color.hexValue}}
          data-color-value={color.value}
          data-color-hexvalue={color.hexValue}
          data-color-name={color.name}></div>
        {this.renderColorInfo(color)}
      </div>
    )
  }

  getClassesForComponent () {
    let classes = [styles.Float]

    if (this.options.swatchShape === 'CIRCLE') {
      classes.push(styles.Float__swatchShapeCircle)
    } else if (this.options.swatchShape === 'RECT') {
      classes.push(styles.Float__swatchShapeRect)
    } else {
      classes.push(styles.Float__swatchShapeSquare)
    }

    if (this.options.swatchShapeRounded) {
      classes.push(styles.Float__swatchShapeRounded)
    }

    if (!this.options.showName) {
      classes.push(styles.Float__noInfo)
    }

    if (this.options.colorsPerRow === 1) {
      classes.push(styles.Float__singleColor)
    }

    if (this.options.fixedSize) {
      classes.push(styles.Float__fixedSize)
    }

    let backgroundIsDark = colorIsDark(this.options.backgroundColor)
    if (backgroundIsDark) {
      classes.push(styles.Float__dark)
    }

    if (this.options.view === 'SamplesView') {
      classes.push(styles.Float__SamplesView)
    }

    return classes.join(' ')
  }

  render () {
    return (
      <div className={this.getClassesForComponent() + ' OctPalette'} data-palette-title={this.props.name} style={{backgroundColor: this.options.backgroundColor}}>
        <div className={styles.Float_header}>{this.props.name}</div>

        <div className={styles.Float_items}>
          {this.props.entries.map(this.renderEntry)}
        </div>
      </div>
    )
  }
};

Float.propTypes = {
  entries: React.PropTypes.array.isRequired,
  name: React.PropTypes.string,
  options: React.PropTypes.shape({
    showName: React.PropTypes.bool,
    showValue: React.PropTypes.bool,
    colorsPerRow: React.PropTypes.number,
    swatchShape: React.PropTypes.string,
    swatchShapeRounded: React.PropTypes.bool,
    backgroundColor: React.PropTypes.string,
    fixedSize: React.PropTypes.bool,
    view: React.PropTypes.string
  })
}
