import React from 'react'
import BaseComponent from '../BaseComponent/BaseComponent'
import styles from './Grid.scss'
import { colorIsDark } from '../../lib/luminance'
import { parseColor } from '../../lib/oco-utils'

export default class Grid extends BaseComponent {
  constructor (props) {
    super(props)

    this.options = props.options

    this.renderColor = this.renderColor.bind(this)
    this.renderPalette = this.renderPalette.bind(this)
  }

  renderColorInfo (color) {
    let showName = this.options.showName
    let showValue = this.options.showValue

    let colorInfo = ''
    let colorValue = ''

    if (showName) {
      colorInfo = <div className={styles.Color_info_name}>{color.name}</div>
    }
    if (showValue) {
      colorValue = <div className={styles.Color_info_value}>{color.value}</div>
    }
    if (colorInfo || colorValue) {
      colorInfo =
        <div className={styles.Color_info}>
          {colorInfo}
          {colorValue}
        </div>
    }
    return colorInfo
  }

  renderColor (entry, index) {
    let color = parseColor(entry)
    let colorWidth = (100 / this.options.colorsPerRow) + '%'

    return (
      <div className={this.getClassesForColor(color, styles, index)} key={index} style={{width: colorWidth}}>
        <div className={styles.Color_swatch + ' OctColor'}
          style={{backgroundColor: color.hexValue}}
          data-color-value={color.value}
          data-color-hexvalue={color.hexValue}
          data-color-name={color.name}>
          {this.renderColorInfo(color)}
        </div>
      </div>
    )
  }

  getClassesForComponent () {
    let classes = [styles.Grid]

    if (this.options.swatchShape === 'SQUARE') {
      classes.push(styles.Grid__swatchShapeSquare)
    } else {
      classes.push(styles.Grid__swatchShapeRect)
    }

    if (this.options.groupMargin) {
      classes.push(styles.Grid__groupMargin)
    }

    if (this.options.groupShapeRounded) {
      classes.push(styles.Grid__groupShapeRounded)
    }

    if (!this.options.showName) {
      classes.push(styles.Grid__noInfo)
    }

    if (this.options.colorsPerRow === 1) {
      classes.push(styles.Grid__singleColor)
    }

    let backgroundIsDark = colorIsDark(this.options.backgroundColor)
    if (backgroundIsDark) {
      classes.push(styles.Grid__dark)
    }

    if (this.options.view === 'GridView') {
      classes.push(styles.Grid__GridView)
    }
    return classes.join(' ')
  }

  render () {
    return (
      <div className={this.getClassesForComponent() + ' OctPalette'} data-palette-title={this.props.name} style={{backgroundColor: this.options.backgroundColor}}>
        <div className={styles.Grid_header}>{this.props.name}</div>

        <div className={styles.Grid_items}>
          {this.props.entries.map(this.renderEntry)}
        </div>
      </div>
    )
  }
}

Grid.propTypes = {
  entries: React.PropTypes.array.isRequired,
  name: React.PropTypes.string,
  options: React.PropTypes.shape({
    showName: React.PropTypes.bool,
    showValue: React.PropTypes.bool,
    colorsPerRow: React.PropTypes.number,
    groupShapeRounded: React.PropTypes.bool,
    groupMargin: React.PropTypes.bool,
    swatchShape: React.PropTypes.string,
    backgroundColor: React.PropTypes.string,
    view: React.PropTypes.string
  })
}
