import React from 'react'
import BaseComponent from '../BaseComponent/BaseComponent'
import styles from './Table.scss'
import { colorIsDark } from '../../lib/luminance'
import { parseColor } from '../../lib/oco-utils'

export default class Table extends BaseComponent {
  constructor (props) {
    super(props)

    this.options = props.options

    this.renderColor = this.renderColor.bind(this)
    this.renderPalette = this.renderPalette.bind(this)
  }

  renderColorInfo (color) {
    let showValue = this.options.showValue

    let colorValue = ''
    if (showValue) {
      colorValue = <div className={styles.Color_info_value}>{color.value}</div>
    }

    let colorInfo =
      <td className={styles.Color_info}>
        <div className={styles.Color_info_name}>{color.name}</div>
        {colorValue}
      </td>

    return colorInfo
  }

  renderColor (entry, index) {
    let color = parseColor(entry)

    return (
      <tr className={this.getClassesForColor(color, styles, index)} key={index}>
        <td className={styles.Color_swatch_container}>
          <div className={styles.Color_swatch + ' OctColor'}
            style={{backgroundColor: color.hexValue}}
            data-color-value={color.value}
            data-color-hexvalue={color.hexValue}
            data-color-name={color.name}></div>
        </td>
        {this.renderColorInfo(color)}
      </tr>
    )
  }

  getClassesForComponent () {
    let classes = [styles.Table]

    if (this.options.swatchShape === 'STRETCHED') {
      classes.push(styles.Table__swatchShapeStreched)
    } else if (this.options.swatchShape === 'SQUARE') {
      classes.push(styles.Table__swatchShapeSquare)
    } else {
      classes.push(styles.Table__swatchShapeCircle)
    }

    if (this.options.swatchShapeRounded) {
      classes.push(styles.Table__swatchShapeRounded)
    }

    let backgroundIsDark = colorIsDark(this.options.backgroundColor)
    if (backgroundIsDark) {
      classes.push(styles.Table__dark)
    }
    return classes.join(' ')
  }

  render () {
    return (
      <div className={this.getClassesForComponent() + ' OctPalette'} data-palette-title={this.props.name} style={{backgroundColor: this.options.backgroundColor}}>
        <div className={styles.Table_header}>{this.props.name}</div>

        <div className={styles.Table_items}>
          <table><tbody>
          {this.props.entries.map(this.renderEntry)}
          </tbody></table>
        </div>
      </div>
    )
  }
}

Table.propTypes = {
  entries: React.PropTypes.array.isRequired,
  name: React.PropTypes.string,
  options: React.PropTypes.shape({
    showValue: React.PropTypes.bool,
    swatchShape: React.PropTypes.string,
    swatchShapeRounded: React.PropTypes.bool,
    backgroundColor: React.PropTypes.string
  })
}
