/* global localStorage */
import React from 'react'
import {parse} from 'opencolor'

import ColorPicker from '../ColorPicker/ColorPicker'
import Error from '../Error/Error'
import { getOcoPalettes } from './oco-palettes'

export default class TestingApp extends React.Component {
  constructor (props) {
    super(props)
    let ocoFiles = getOcoPalettes()
    let ocoFilesKeys = Object.keys(ocoFiles)

    let currentFile = localStorage.getItem('currentFile')

    if (currentFile && ocoFilesKeys.indexOf(currentFile) > -1) {
    } else {
      currentFile = ocoFilesKeys[0]
    }

    this.state = {
      ocoFiles: ocoFiles,
      options: ocoFilesKeys,
      currentFile: currentFile
    }

    this.switchOcoFile = this.switchOcoFile.bind(this)
  }

  switchOcoFile (e) {
    localStorage.setItem('currentFile', e.target.value)
    this.setState({currentFile: e.target.value})
  }

  getParsedOco () {
    return parse(this.state.ocoFiles[this.state.currentFile])
  }

  renderFileSelector () {
    var options = this.state.options.map(function (option) {
      return (
        <option key={option} value={option}>
          {option}
        </option>
      )
    })

    return (
      <select onChange={this.switchOcoFile} value={this.state.currentFile}>
        {options}
      </select>
    )
  }

  renderColorPicker () {
    try {
      var parsedOco = this.getParsedOco()

      return (
        <ColorPicker parsedOco={parsedOco} style={{border: '1px solid #ccc'}} />
      )
    } catch (err) {
      return (
        <Error error={err} />
      )
    }
  }

  render () {
    return (
      <div>
        {this.renderFileSelector()}
        {this.renderColorPicker()}
      </div>
    )
  }
}
