var Palette = {};

(function(self) {
  self.init = init;
  self.onLoad = onLoad;
  self.onEmpty = onEmpty;
  self.setColorFromPicker = setColorFromPicker;

  var lumaThreshold = 170;

  function onLoad(data) {
    OcoRenderer.destroy($('#oco-renderer-outlet')[0]);
    OcoRenderer.parseAndRender(data, $('#oco-renderer-outlet')[0]);
  }

  function onEmpty() {
  }

  function setColorFromPicker(colorHex) {
    colorHex = colorHex.toUpperCase()

    if ($('.PickedColor').data('color-hexvalue') === colorHex) {
      return;
    }

    var $colors = $('.OctColor[data-color-hexvalue="' + colorHex + '"]');
    if (!$colors.size()) {
      $(".Color-swatch .SelectedColor").remove();
      $(".OctColor__isSelected").removeClass("OctColor__isSelected")
      setPickedColor(null, "Unknown", colorHex, colorHex);
      return;
    }
    var paletteName = $colors.first().parents('.OctPalette').data('palette-title');
    setPickedColor(paletteName, $colors.first().data('color-name'), $colors.first().data('color-value'), colorHex);
    addCheckmark($colors, colorHex);

  }

  function getLuminanceForHex(colorHex) {
    var c = colorHex.substring(1);      // strip #
    var rgb = parseInt(c, 16);   // convert rrggbb to decimal
    var r = (rgb >> 16) & 0xff;  // extract red
    var g = (rgb >>  8) & 0xff;  // extract green
    var b = (rgb >>  0) & 0xff;  // extract blue

    var luma = 0.2126 * r + 0.7152 * g + 0.0722 * b; // per ITU-R BT.709

    return luma;
  }

  function setColorsForCopyButton(colorHex) {
    var elm = $('.PickedColorCopyButton');
    var luma = getLuminanceForHex(colorHex);

    elm.removeClass('NoColorSelected');
    elm.css('background', colorHex);

    if (luma > lumaThreshold) {
      elm.removeClass("DarkColorSelected").addClass("LightColorSelected");
    } else {
      elm.removeClass("LightColorSelected").addClass("DarkColorSelected");
    }
  }

  function addCheckmark(colorElm, colorHex) {
    $(".OctColor__isSelected").removeClass("OctColor__isSelected")

    colorElm.each(function() {
      var $elm = $(this);
      $elm.parents(".OctColor__dark").first().addClass("OctColor__isSelected");
      $elm.parents(".OctColor__light").first().addClass("OctColor__isSelected");
    });
  }

  function setPickedColor(paletteName, colorName, colorValue, colorHex) {
    resetCopyButton();

    $('.PickedColor').data('color-hexvalue', colorHex);
    $('.PickedColor .PickedColor-swatch').css('background', colorHex);

    $('.PickedColorName').removeClass('NoColorSelected');
    var pickerColorName = colorName;
    if (paletteName) {
      pickerColorName = pickerColorName + ' (' + paletteName + ')'
    }
    $('.PickedColorName').text(pickerColorName);

    var pickedColorInfo;
    if (colorValue.toUpperCase() === colorHex) {
      pickedColorInfo = colorHex;
    } else {
      pickedColorInfo = colorValue + ' · ' + colorHex
    }
    $('.PickedColorHex').text(pickedColorInfo);

    setColorsForCopyButton(colorHex);
  }

  function colorPicked(colorElm) {
    var colorName = colorElm.data('color-name');
    var colorValue = colorElm.data('color-value');
    var colorHex = colorElm.data('color-hexvalue');
    var paletteName = colorElm.parents('.OctPalette').data('palette-title');

    addCheckmark(colorElm, colorHex);
    setPickedColor(paletteName, colorName, colorValue, colorHex);

    // send color to picker
    Picker.colorPicked(colorHex);
  }

  function resetCopyButton() {
    $('.PickedColorCopyButton .DefaultState').show();
    $('.PickedColorCopyButton .CopiedState').hide();
  }

  function copiedCopyButton() {
    $('.PickedColorCopyButton .DefaultState').hide();
    $('.PickedColorCopyButton .CopiedState').show();
  }

  function copyToClipboardClicked() {
    var colorHex = $('.PickedColor').data('color-hexvalue');

    if (colorHex) {
      copiedCopyButton();
      setTimeout(function(){ resetCopyButton(); }, 1500);

      Picker.copyToClipboard(colorHex);
    }
  }

  function init() {
    $('body').on('click', '.OctColor', function() {
      colorPicked($(this));
    });

    $('body').on('click', '.PickedColorCopyButton', function() {
      copyToClipboardClicked();
    });

    $('body').on('mouseover', '.OctColor', function() {
      $(this).addClass('hover');
    });

    $('body').on('mouseout', '.OctColor', function() {
      $(this).removeClass('hover');
    });

    $('body').on('mouseover', '.PickedColorCopyButton', function() {
      $(this).addClass('hover');
    });

    $('body').on('mouseout', '.PickedColorCopyButton', function() {
      $(this).removeClass('hover');
    });
  }

})(Palette);
