var Picker = {};

(function(self) {
  self.init = init;
  self.copyToClipboard = copyToClipboard;
  self.load = load;
  self.colorPicked = colorPicked;

  var bridge;
  var mousePos;
  var lastHoveredElement;

  function init(callback) {
    if (window.WebViewJavascriptBridge) {
      bridge = WebViewJavascriptBridge;
      bridge.init(function(message, responseCallback) {});
      registerHandlers();
      callback();
    }
    else {
      document.addEventListener('WebViewJavascriptBridgeReady', function() {
        bridge = WebViewJavascriptBridge;
        bridge.init(function(message, responseCallback) {});
        registerHandlers();
        callback();
      }, false);
    }
  }

  function load(palette) {
    if (palette) {
      Palette.onLoad(palette.data);
    } else {
      Palette.onEmpty();
    }
  }

  function colorPicked(hex) {
    bridge.callHandler('colorPicked', hex);
  }

  function copyToClipboard(hex) {
    bridge.callHandler('copyToClipboard', hex);
  }

  function registerHandlers() {
    bridge.registerHandler('load', function(paletteData) {
      load(paletteData);
    });

    bridge.registerHandler('setColorFromPicker', function(hex) {
      Palette.setColorFromPicker(hex);
    });

    bridge.registerHandler('moveMouse', function(position) {
      mousePos = position;

      //get hovered element
      var hoveredElement = $(document.elementFromPoint(mousePos.x, mousePos.y));

      //if hover element is different from last hovered element
      if (!hoveredElement.is(lastHoveredElement)) {
        //if last element is set
        if (lastHoveredElement) {
          //trigger event on element
          simulateEventOnElement('mouseleave', lastHoveredElement);
        }

        //trigger event on element
        simulateEventOnElement('mouseenter', hoveredElement);

        //assign current element to last element
        lastHoveredElement = hoveredElement;
      }
    });

    bridge.registerHandler('mouseExited', function(position) {
      if (lastHoveredElement) {
        simulateEventOnElement('mouseleave', lastHoveredElement);
      }
    });
  }

  function simulateEventOnElement(eventType, element) {
    //create event
    var e = jQuery.Event(eventType);

    //trigger event on element
    element.trigger(e);
  }

  function simulateEventOnPos(eventType, x, y) {
    //get element at position
    var element = $(document.elementFromPoint(x, y));

    //create event
    var e = jQuery.Event(eventType);

    //trigger event on element
    element.trigger(e);
  }
})(Picker);
